#pragma once

#include <boost/filesystem/fstream.hpp>    // for ifstream
#include <boost/filesystem/operations.hpp> // for exists
#include <boost/filesystem/path.hpp>       // for path
#include <fstream>                         // for ifstream, size_t
#include <memory>                          // for shared_ptr
#include <string>                          // for string

namespace caosdb::transaction {
using boost::filesystem::exists;
using boost::filesystem::ifstream;
using boost::filesystem::path;

class FileReader final {
public:
  FileReader(boost::filesystem::path filename);

  ~FileReader() = default;

  FileReader(const FileReader &) = delete;
  FileReader &operator=(const FileReader &) = delete;

  FileReader(FileReader &&) = default;
  FileReader &operator=(FileReader &&) = default;

  unsigned long long fileSize() const { return size_; }

  std::size_t read(std::string &buffer);

private:
  void openFile();

  std::ifstream stream_;
  boost::filesystem::path filename_;
  unsigned long long size_;
};

} // namespace caosdb::transaction
